@@ -32,6 +32,7 @@ We also provide some handy utility functions:
 from __future__ import absolute_import
 
 import sys
+import time
 
 from esky.util import lazy_import
 
@@ -48,6 +49,14 @@ def pickle():
         import pickle
     return pickle
 
+@lazy_import
+def threading():
+    try:
+        import threading
+    except ImportError:
+        threading = None
+    return threading
+
 
 if sys.platform == "win32":
     @lazy_import
@@ -93,11 +102,15 @@ def b(data):
 
 
 class SudoProxy(object):
-    """Object method proxy with root privileges."""
+    """Object method proxy with root privileges.
+
+    This class creates a copy of an object whose methods can be executed
+    with root privileges.
+    """
 
     def __init__(self,target):
         #  Reflect the 'name' attribute if it has one, but don't worry
-        #  if not.  This helps SudoProxy be re-used on other clases.
+        #  if not.  This helps SudoProxy be re-used on other classes.
         try:
             self.name = target.name
         except AttributeError:
@@ -110,9 +123,33 @@ class SudoProxy(object):
         (self.proc,self.pipe) = spawn_sudo(self)
         if self.proc.poll() is not None:
             raise RuntimeError("sudo helper process terminated unexpectedly")
-        if self.pipe.read() != b("READY"):
+        #  If threading is available, run a background thread to monitor
+        #  the sudo process.  If it dies, terminate things immediately.
+        if threading:
+            self._do_monitor_proc = True
+            monitor_thread = threading.Thread(target=self._monitor_proc)
+            monitor_thread.daemon = True
+            monitor_thread.start()
+        #  Try to read initialisation message from the pipe.
+        #  If this fails, the helper program must have died.
+        try:
+            msg = self.pipe.read()
+        except EOFError:
+            msg = b("")
+        if msg != b("READY"):
             self.close()
             raise RuntimeError("failed to spawn helper app")
+        if threading:
+            self._do_monitor_proc = False
+            monitor_thread.join()
+
+    def _monitor_proc(self):
+        while self._do_monitor_proc:
+            if self.proc.poll() is not None:
+                self.pipe._recover()
+                self.pipe.close()
+                break
+            time.sleep(0)
 
     def close(self):
         self.pipe.write(b("CLOSE"))
@@ -230,7 +267,7 @@ def allow_from_sudo(*argtypes,**kwds):
     through the sudo proxy: allowing it via this decorator, and actually 
     passing on the call to the proxy object.  I have no intention of making
     this any more hidden, because the fact that a method can have escalated
-    privileges is somethat that needs to be very obvious from the code.
+    privileges is something that that needs to be very obvious from the code.
     """
     def decorator(func):
         func._esky_sudo_argtypes = argtypes
@@ -272,12 +309,22 @@ def _get_sudo_iterator(obj,methname):
     return False
 
 def _get_mro(obj):
+    """Get the method resolution order for an object.
+
+    In other words, get the list of classes what are used to look up methods
+    on the given object, in the order in which they'll be consulted.
+    """
     try:
         return obj.__class__.__mro__
     except AttributeError:
         return _get_oldstyle_mro(obj.__class__,set())
 
 def _get_oldstyle_mro(cls,seen):
+    """Get the method resolution order bor an old-style class.
+
+    This is essentially a bottom-up left-to-right iteration of all the
+    superclasses.
+    """
     yield cls
     seen.add(cls)
     for base in cls.__bases__:
