@@ -396,7 +396,7 @@ class TestEsky(unittest.TestCase):
     it with the contents of esky.__doc__.
     """
     dirname = os.path.dirname
-    readme = os.path.join(dirname(dirname(dirname(__file__))),"README.txt")
+    readme = os.path.join(dirname(dirname(dirname(__file__))),"README.rst")
     if not os.path.isfile(readme):
         f = open(readme,"wb")
         f.write(esky.__doc__.encode())
